CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  ludcmp.f   - Obtained from "Numerical Recipes"
C  
C  Takes an NxN matrix A of physical dimension NP. A new A matrix
C  is returned along with INDX to be used by the LU back
C  substitution subroutine.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

       subroutine ludcmp( A,N,NP,INDX,D )
       implicit none
       integer NMAX,TINY,NP
       parameter (NMAX=101,TINY=1.0E-20)
       integer INDX(NP),N,i,j,k,imax
       real*8 A(NP,NP),VV(NMAX),aamax,sum,dum,D

       d=1.0

       do i=1,N
         aamax = 0.0
         
         do j=1,N
C	   write(6,*) "A(",i,",",j,") = ",A(i,j)
           if ( abs(A(i,j)) .gt. aamax ) aamax = abs(A(i,j))
         enddo

         if ( aamax .eq. 0.0 ) then
	   write(6,*) "*****************************"
           write(6,*) "      Singular matrix."
           write(6,*) "*****************************"
           stop
         endif

         VV(i) = 1.0/aamax
       enddo


       do j=1,N
         do i=1,j-1
           sum=A(i,j)
    
           do k=1,i-1
             sum = sum - A(i,k)*A(k,j)
           enddo
         
           A(i,j) = sum
         enddo

         aamax = 0.0
 
         do i=j,N
           sum=A(i,j)

           do k=1,j-1
             sum = sum - A(i,k)*A(k,j)
           enddo
 
           A(i,j) = sum

           dum = VV(i)*abs(sum)

           if ( dum .ge. aamax ) then
             imax = i
             aamax = dum
           endif
         enddo

         if ( j .ne. imax ) then
           do k=1,N
             dum = A(imax,k)
             A(imax,k) = A(j,k)
             A(j,k) = dum
           enddo
   
           d = -d

           VV(imax) = VV(j)
         endif




         INDX(j) = imax

         if ( A(j,j) .eq. 0.0 ) A(j,j) = TINY

         if ( j .ne. N ) then
           dum = 1.0 / A(j,j)

           do i=j+1,N
             A(i,j) = A(i,j)*dum
           enddo
         endif
       enddo

       return
       end

 
