C       10        20        30        40        50        60        70
C23456789_C23456789_C23456789_C23456789_C23456789_C23456789_C23456789_

C *******************************************************************
C *                                                                 *
C *  Program:  thermal_1d.f                                         *
C *                                                                 *
C *  Author:   Mariusz Zaczek  (zaczek@uiuc.edu)                    *
C *  Date:     May 7,2000                                           *
C *                                                                 *
C *******************************************************************

       program thermal_1d
       implicit none

       integer max_nodes,max_elem
       parameter(max_nodes=101,max_elem=100)

       integer numsteps,step_incr,numelem,numbcs,
     >        imposed_node(max_nodes)

       real*8 delta_t,K(max_nodes,max_nodes),C(max_nodes,max_nodes),
     >        R(max_nodes),length(max_elem),density(max_elem),
     >        k_val(max_elem),c_val(max_elem),Q(max_elem),
     >        C_plus(max_nodes,max_nodes),T(max_nodes),
     >        C_minus(max_nodes,max_nodes),beta,
     >        imposed_T(max_nodes)

C      ! call subroutine 'file_input' to read input data from file
       call  file_input( numsteps,step_incr,delta_t,beta, 
     >       length,density,k_val,c_val,Q,numelem,numbcs,
     >       imposed_node,imposed_T ) 

C      ! Calculate Global [K] and [C] matrices and Global [R] vector
       call  assemble( K,C,R,numelem,length,density,k_val,c_val,Q,
     >       delta_t,C_plus,C_minus,beta,numbcs,
     >       imposed_node,imposed_T )

C      ! Begin solution loop
       call   solution( delta_t,numsteps,step_incr,C_minus,
     >            C_plus,T,R,numelem,numbcs,imposed_node,imposed_T )

       end    ! End of thermal_1d program


C *******************************************************************
C   
C   solution - loop over the timesteps prescribed
C 
C *******************************************************************
       subroutine solution( delta_t,numsteps,step_incr,C_minus,
     >            C_plus,T,R,numelem,numbcs,imposed_node,imposed_T )

       integer max_nodes,max_elem
       parameter(max_nodes=101,max_elem=100)

       integer i,numelem,numsteps,step_incr,indx(max_nodes),numbcs,
     >    imposed_node(max_nodes),k,m

       real*8 delta_t,T(max_nodes),R(max_nodes),b(max_nodes),d_info,
     >     C_plus(max_nodes,max_nodes),C_minus(max_nodes,max_nodes),
     >     A(max_nodes,max_nodes),imposed_T(max_nodes)

C      ! Initialize the first Temperature at time t=0
       do i=1,numelem+1
         T(i) = 0.0
         b(i) = 0.0
       enddo

C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C      ! Start SOLUTION loop - solve linear system: Ax=b
C CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       do i=1,numsteps

C        ! Compute new {b} vector
         call calc_b( C_minus,T,R,numelem,b,numbcs,imposed_node,
     >          imposed_T )
         do j=1,numelem+1
	   write(99,'(1X,11E11.3e2,1A,E16.6e2)') (C_plus(j,k),k=1,11)
     >       ,"  |  ",b(j)
         enddo
         write(99,*)
         do j=1,numelem+1
           write(99,'(1X,11E11.3e2)') (C_minus(j,k),k=1,11)
          enddo
	 write(99,*)
	 write(99,*)

C        ! Must find another linear solver...one that does not
C        ! destroy the [C_plus] matrix and {R} vector
         do k=1,numelem+1
           do m=1,numelem+1
             A(k,m) = C_plus(k,m)
           enddo
         enddo

C        ! Solve the linear system Ax=b which is represented by:
C        ! C_plus*T(n+1) = R ... want to solve for new T = T(n+1)
         call ludcmp( A,numelem+1,101,indx,d_info )
         call lubksb( A,numelem+1,101,indx,b )

C        ! Update the new temperature: T = b
	 do k=1,numelem+1
           T(k) = b(k)
         enddo

         write(99,'(11E12.4e3)')T(1),T(2),T(3),T(4),T(5),T(6),
     >   T(7),T(8),T(9),T(10),T(11)
         write(99,*)
         write(99,*)

         if ( mod(i,step_incr) .eq. 0 ) then
	   write(77,*) i
           do k=1,numelem+1
     	     write(77,'(F12.4)')  T(k)
           enddo
         endif

       enddo    ! End of SOLUTION loop


       write(99,*)
       write(6,*)
       write(99,*) " ******** FINAL TEMPERATURE ******* "
       write(6,*)  " ******** FINAL TEMPERATURE ******* "
       write(99,*)
       write(6,*)
       do i=1,numelem+1
         write(99,'(1X,E17.10E3)') b(i)
	 write(6,'(1X,E17.10E3)') b(i)
       enddo

       return
       end         ! End of "solution" subroutine



C *******************************************************************
C   
C   calc_b - calculate the b vector of linear system Ax=b
C 
C *******************************************************************
       subroutine calc_b( C_minus,T,R,numelem,b,numbcs,imposed_node,
     >              imposed_T )
 
       integer max_nodes
       parameter(max_nodes=101)
       integer i,j,numelem,numbcs,imposed_node(max_nodes)

       real*8 R(max_nodes),T(max_nodes),C_minus(max_nodes,max_nodes),
     >        real_1,b(max_nodes),imposed_T(max_nodes)
       
C      ! Calculate [Cminus]{T} + {R}
       do i=1,numelem+1
         real_1 = 0.0
         do j=1,numelem+1
           real_1 = real_1 + C_minus(i,j)*T(j)
         enddo
         b(i) = R(i) + real_1 
       enddo


C      ! Impose the imposed temperatures
       do i=1,numbcs
         b(imposed_node(i)) = imposed_T(i)
       enddo

       return
       end         ! End of "calc_b" subroutine 


C *******************************************************************
C   
C   assemble - assemble the global matrices K,M and vector R
C 
C *******************************************************************
       subroutine assemble( K,C,R,numelem,length,density,k_val,
     >            c_val,Q,delta_t,C_plus,C_minus,beta,numbcs,
     >            imposed_node,imposed_T )

       integer max_nodes,max_elem
       real*8 one_sixth
       parameter(max_nodes=101,max_elem=100,one_sixth=0.1666667)
       integer i,j,numelem,numbcs,int_1,imposed_node(max_nodes)

       real*8 delta_t,K(max_nodes,max_nodes),C(max_nodes,max_nodes),
     >        R(max_nodes),length(max_elem),density(max_elem),
     >        k_val(max_elem),c_val(max_elem),Q(max_elem),c_const,
     >        k_const,q_const,C_plus(max_nodes,max_nodes),
     >        C_minus(max_nodes,max_nodes),beta,
     >        imposed_T(max_nodes)

C      ! Clear the matrices and vector initially
       do i=1,numelem+1
         do j=1,numelem+1
           K(i,j) = 0.0
           C(i,j) = 0.0
         enddo
         R(i) = 0.0
       enddo

C      ! Assemble the global matrices and vectors
       do i=1,numelem
C        ! Assemble the [K] matrix
         k_const = k_val(i)/length(i)
         K(i  ,i  ) = K(i  ,i  ) + k_const
         K(i+1,i+1) = K(i+1,i+1) + k_const
         K(i  ,i+1) = K(i  ,i+1) - k_const
         K(i+1,i  ) = K(i+1,i  ) - k_const

C        ! Assemble the [C] Matrix
         c_const = density(i)*c_val(i)*length(i)*one_sixth
         C(i  ,i  ) = C(i  ,i  ) + c_const*2.0
         C(i+1,i+1) = C(i+1,i+1) + c_const*2.0
         C(i  ,i+1) = C(i  ,i+1) + c_const
         C(i+1,i  ) = C(i+1,i  ) + c_const 

C        ! Assemble the {R} vector
         q_const = Q(i)*length(i)*0.5
         R(i  ) = R(i  ) + q_const
         R(i+1) = R(i+1) + q_const
       enddo


C      ! To save calculations, precalculate the constant summed
C      ! matrices: ([C]/delta_t + beta*[K])^-1   and
C      ! ([C]/delta_t - (1-beta)*[K])
       do i=1,numelem+1
         do j=1,numelem+1
            C_plus(i,j) = C(i,j)/delta_t + beta*K(i,j)
           C_minus(i,j) = C_plus(i,j) - K(i,j)
         enddo
       enddo

C      ! Loop over imposed temperatures... 
C      ! and st the corresponding rows of the A matrix to have a 1
C      ! for the imposed node and 0 elsewhere... and then also
C      ! add the imposed temp to the right hand side.
       do i=1,numbcs
         int_1 = imposed_node(i)

C        ! Set everything in row to 0.0
         do j=1,numelem+1
           C_plus(int_1,j) = 0.0
         enddo

C        ! Set imposed node to 1.0
         C_plus(int_1,int_1) = 1.0

       enddo


       return
       end         ! End of "assemble" subroutine


C *******************************************************************
C
C   file_input - read the input file
C
C *******************************************************************
       subroutine file_input( numsteps,step_incr,delta_t,beta,
     >            length,density,k_val,c_val,Q,numelem,numbcs,
     >            imposed_node,imposed_T )

       integer max_nodes,max_elem
       parameter(max_nodes=101,max_elem=100)
       integer numsteps,step_incr,numelem,numbcs

       integer i,int_1,int_2,int_3,inlen,imposed_node(max_nodes)
       character*20 infile

       real*8 delta_t,beta,length(max_elem),density(max_elem),
     >        k_val(max_elem),c_val(max_elem),Q(max_elem),
     >        imposed_T(max_nodes),
     >        total_len,len_small,len_large

       real*8 real_1,real_2,real_3,real_4,real_5,real_6

C      ! Initialize the small and large value of element lengths
       len_small = 99999.9
       len_large = 0.0
       real_6 = 0.0

C      ! Open file for reading
       open(7,file='input_thermal',status='old')

 10    format(A100)
 20    format(a20)

C      # File name for data output
       read(7,10)                 ! comments for prog. name
       read(7,20) infile          ! file name

C      ! Prepare for output file
       inlen = 20
       do i = 1,20
         if (infile(i:i).ne.' ') then
           inlen = i
         endif
       enddo
       
C      ! Open problem summary file  
       open(99,file=''//infile(1:inlen)//'.res',status='unknown')
       open(77,file='input_display_thermal',status='unknown')

C      # <num tsteps> <tstep incr> <delta_t> <Beta>
       read(7,10)                 ! comments for timestep info
       read(7,*) numsteps,step_incr,delta_t,beta

C      # <total len(m)> <num elem.>
       read(7,10)                 ! comments for lengthC     
       read(7,*) total_len,numelem
         write(77,*) numsteps/step_incr
         write(77,*) numelem+1

C      # Number regions for element sizes
       read(7,10)                 ! comments for # length regions
       read(7,*) int_1

C      # If diff.:<From elem #><To elem #><l><k_val><rho><c_val><Q>
       read(7,10)                 ! comments for element lengths
             
       do i=1,int_1
         read(7,*) int_2,int_3,real_1,real_2,real_3,real_4,real_5
             
C        ! Update the smallest element length
         if ( real_1 .lt. len_small ) then
           len_small = real_1
         endif
           
C        ! Update the largest element length
         if ( real_1 .gt. len_large ) then
           len_large = real_1
         endif 

C        ! Assign the size of the elements based on input data
         do j=int_2,int_3
           length(j) = real_1
            k_val(j) = real_2
          density(j) = real_3
            c_val(j) = real_4
                Q(j) = real_5

C          ! Update overall length calculation   
           real_6 = real_6 + real_1
         enddo     ! end 'do j=int_2,int_3'
       enddo     !  end 'do i=1,int_1'
             
C      ! Check if all the elements add up to the specified length
       if ( abs(real_6 - total_len) .gt. 0.00001 ) then
         write(6,*) "Error(1): Total specified length,",total_len,
     >              " , does not match the sum of the lengths",
     >              real_6  
         stop
       endif

C      # Boundary conditions: <number bcs> (applied at nodes)
       read(7,10)                 ! comments for BCs
       read(7,*) numbcs

C      # BCs: <node #> <Temperature Imposed>
       read(7,10)                 ! comments for bc values read
       do i=1,numbcs
         read(7,*) imposed_node(i),imposed_T(i)
       enddo

C      ! Close file
       close(7)

C      ! Output pertinent input data to the .res file 
       write(99,*) infile
       write(99,*) 


       return
       end           ! End of "file_input" subroutine






